import os
# from PIL import Image
import cv2
import numpy as np
import pandas as pd
import openpyxl
from openpyxl.drawing.image import Image
from openpyxl.styles import Alignment

from keras.datasets import mnist

# opencv-contrib-python 4.8.1.78
# specify your directory path here
directory_path = "data/MNIST_IMAGE"
folder_name = "MNIST_IMAGE"


def load_mnist(store_image_list, store_data_list, start_number, end_number):
    # 訓練データ画像をと正解データを読み込み
    (X_train, Y_train), (X_test, y_test) = mnist.load_data()
    # 指定（start_numberからend_numberまで）する訓練データ画像と正解データのリストを作成する
    for i in range(start_number, end_number+1):
        store_image_list.append(X_train[i])
        store_data_list.append(Y_train[i])
    # 指定（start_numberからend_numberまで）する訓練データ画像を作成し保存する
    number = start_number
    path_dir = 'data/MNIST_IMAGE'
    if not os.path.exists(path_dir):
        os.mkdir(path_dir)
    for image in store_image_list:
        correct_answer_number = Y_train[number]
        file_name = 'data/MNIST_IMAGE/image_file_' + str(number) + '_' + str(correct_answer_number) + '.png'
        cv2.imwrite(file_name, image)
        number += 1


def insert_image_colum_to_exel(exel_work_sheet, image_column):
    # get number of columns
    columns_len = len(list(exel_work_sheet.rows))

    # Paste images
    for i in range(1, columns_len):
        index = i + 1
        cell_name = image_column + str(index)
        target_cell = exel_work_sheet[cell_name]
        try:
            # Path情報を元に画像の所得
            target_image = Image(target_cell.value)
            # 画像を貼り付ける
            exel_work_sheet.add_image(target_image, cell_name)
            # セルの高さを調整（画像のピクセル単位からExcelのポイント単位に変換して渡す
            exel_work_sheet.row_dimensions[index].height = float(target_image.height / 1.33)
            # 画像の貼り付けに使った画像ファイルのPATH情報を削除しておく
            target_cell.value = ""
        except FileNotFoundError:
            # 画像ファイルが見つからない場合はスキップ
            print("not found image")
            pass


def get_cosine_similarity(path_list, result_list):
    # calculating cosine similarity between grayscale images
    # get the first image as reference image
    ref_img = cv2.imread(path_list[0], cv2.IMREAD_COLOR)
    ref_img_grayscale = 0.299 * ref_img[:, :, 2] + 0.587 * ref_img[:, :, 1] + 0.114 * ref_img[:, :, 0]
    for file_path in path_list[1:]:
        file_name = os.path.basename(file_path)
        target_img = cv2.imread(file_path, cv2.IMREAD_COLOR)
        target_img_grayscale = 0.299 * target_img[:, :, 2] + 0.587 * target_img[:, :, 1] + 0.114 * target_img[:, :, 0]
        a = ref_img_grayscale.ravel()
        b = target_img_grayscale.ravel()
        cosine_similarity = np.dot(a, b) / (np.sqrt(np.dot(a, a)) * np.sqrt(np.dot(b, b)))
        result_list.append([file_path, file_name, round(cosine_similarity, 3)])


def get_PSNR_SSIM(path_list, result_list):
    # calculating PSNR and SSIM
    # get the first image as reference image
    ref_img = cv2.imread(path_list[0], cv2.IMREAD_COLOR)
    for file_path in path_list[1:]:
        file_name = os.path.basename(file_path)
        target_img = cv2.imread(file_path, cv2.IMREAD_COLOR)
        PSNR, _ = cv2.quality.QualityPSNR_compute(target_img, ref_img)
        SSIM, _ = cv2.quality.QualitySSIM_compute(target_img, ref_img)
        PSNR_average = (PSNR[0] + PSNR[1] + PSNR[0]) / 3
        SSIM_average = (SSIM[0] + SSIM[1] + SSIM[0]) / 3
        result_list.append([file_path, file_name, round(PSNR_average, 3), round(SSIM_average, 3)])


def sort():
    file_list = os.listdir(directory_path)
    file_list_sorted = sorted(file_list, reverse=True)
    print(file_list_sorted)


def main():
    # 類似度を計算する画像の生成
    store_list_image = []
    store_list_data = []
    load_mnist(store_list_image, store_list_data, 16, 50)
    # 類似度を計算する画像へのパスリスト
    file_path_list = []
    # 画像の類似度データ格納用リスト
    data_list_ps = []
    data_list_cs = []

    file_list = os.listdir(directory_path)
    # If you want to sort in descending order, use reverse=True
    file_list_sorted = sorted(file_list, reverse=False)
    for filename in file_list_sorted:
        file_path = os.path.join(directory_path, filename)
        file_path_list.append(file_path)

    data_list_ps.append([file_path_list[0], os.path.basename(file_path_list[0]), '--Ref--', '--Ref--'])
    data_list_cs.append([file_path_list[0], os.path.basename(file_path_list[0]), '--Ref--'])

    # calculating PSNR and SSIM
    get_PSNR_SSIM(file_path_list, data_list_ps)
    # calculating cosine similarity
    get_cosine_similarity(file_path_list, data_list_cs)

    df_ps = pd.DataFrame(data_list_ps, columns=['File path', 'File name', 'PSNR', 'SSIM'])
    df_cs = pd.DataFrame(data_list_cs, columns=['File path', 'File name', 'Cosine Similarity(grayscale image)'])

    # making Excel file
    # making DataFrame data
    df = pd.merge(df_cs, df_ps)
    df_excel = df.copy()
    df_excel.insert(2, "image", df["File path"])
    df_excel.drop('File path', axis=1, inplace=True)

    # DataFrameデータをExcelファイルに変換
    excel_file_path = folder_name + ".xlsx"
    df_excel.to_excel(excel_file_path, index=False)
    # Excelファイルを作成し読み込み
    execl_book = openpyxl.load_workbook(excel_file_path)
    # Activeシート取得
    exel_work_sheet = execl_book.active
    # 行と列幅及び文字列の位置を調整
    column_width_A = 30
    column_width_B = 10
    exel_work_sheet.column_dimensions['A'].width = column_width_A
    exel_work_sheet.column_dimensions['B'].width = column_width_B
    for row in exel_work_sheet:
        for cell in row:
            cell.alignment = Alignment(vertical='center', horizontal='left')
    # Excelファイルへ画像を貼り付け
    image_column_symbol = "B"  # is row name of Excel sheet
    insert_image_colum_to_exel(exel_work_sheet, image_column_symbol)
    # 保存
    execl_book.save(excel_file_path)

    # making HTML file
    # making DataFrame data
    df_html = df.copy()
    df_html.insert(2, "image", df["File path"].map(lambda s: "<img src='{}' width='200' />".format(s)))
    df_html.drop('File path', axis=1, inplace=True)
    html_reference = """
    <p> Reference image</p>
    """
    reference = df_html[0:1].to_html(
        classes=["table", "table-bordered", "table-hover"], escape=False
    )
    html_reference += reference
    with open('reference.html', 'w') as f:
        f.write(html_reference)

    html_target = """
    <p> Target image</p>
    """
    target = df_html[1:].to_html(
        classes=["table", "table-bordered", "table-hover"], escape=False
    )
    html_target += target
    with open('target.html', 'w') as f:
        f.write(html_target)


if __name__ == '__main__':
    main()
    # store_list_image = []
    # store_list_data = []
    # load_mnist(store_list_image, store_list_data, 0, 19)
# See PyCharm help at https://www.jetbrains.com/help/pycharm/
